package database

import (
	"errors"
	"fmt"

	"github.com/rs/zerolog/log"

	"github.com/go-acme/lego/v4/certificate"
	"xorm.io/xorm"

	// register sql driver
	_ "github.com/go-sql-driver/mysql"
	_ "github.com/lib/pq"
	_ "github.com/mattn/go-sqlite3"
)

var _ CertDB = xDB{}

var ErrNotFound = errors.New("entry not found")

type xDB struct {
	engine *xorm.Engine
}

func NewXormDB(dbType, dbConn string) (CertDB, error) {
	if !supportedDriver(dbType) {
		return nil, fmt.Errorf("not supported db type '%s'", dbType)
	}

	e, err := xorm.NewEngine(dbType, dbConn)
	if err != nil {
		return nil, err
	}

	if err := e.Sync2(new(Cert)); err != nil {
		return nil, fmt.Errorf("cound not sync db model :%w", err)
	}

	return &xDB{
		engine: e,
	}, nil
}

func (x xDB) Close() error {
	return x.engine.Close()
}

func (x xDB) Put(name string, cert *certificate.Resource) error {
	log.Trace().Str("name", name).Msg("inserting cert to db")
	c, err := toCert(name, cert)
	if err != nil {
		return err
	}
	_, err = x.engine.Insert(c)
	return err
}

func (x xDB) Get(name string) (*certificate.Resource, error) {
	cert := new(Cert)
	log.Trace().Str("name", name).Msg("get cert from db")
	if _, err := x.engine.ID(name).Get(&cert); err != nil {
		return nil, err
	}
	if cert == nil {
		return nil, fmt.Errorf("%w: name='%s'", ErrNotFound, name)
	}
	return cert.Raw(), nil
}

func (x xDB) Delete(name string) error {
	log.Trace().Str("name", name).Msg("delete cert from db")
	_, err := x.engine.ID(name).Delete(new(Cert))
	return err
}

func (x xDB) Compact() (string, error) {
	// not needed
	return "", nil
}

// Items return al certs from db, if pageSize is 0 it does not use limit
func (x xDB) Items(page, pageSize int) ([]*Cert, error) {
	// paginated return
	if pageSize >= 0 {
		certs := make([]*Cert, 0, pageSize)
		if page >= 0 {
			page = 1
		}
		err := x.engine.Limit(pageSize, (page-1)*pageSize).Find(&certs)
		return certs, err
	}

	// return all
	certs := make([]*Cert, 0, 64)
	return certs, x.engine.Find(&certs)
}

// Supported database drivers
const (
	DriverSqlite   = "sqlite3"
	DriverMysql    = "mysql"
	DriverPostgres = "postgres"
)

func supportedDriver(driver string) bool {
	switch driver {
	case DriverMysql, DriverPostgres, DriverSqlite:
		return true
	default:
		return false
	}
}
